/*
;;tbcursor.hpp
;;
;;  TBCursor class, inherited from BCursor
;;
*/

#ifdef    I_TBCURSOR
#else
#define   I_TBCURSOR

#include  <kdbf.h>

#include  <stdcpp.hpp>
#include  <tbdataba.hpp>

class TBRecord;

/*
;;class TBCursor
;;  : public BCursor
;;
;;  This class adds the following functionality beyond BCursor:
;;    -Throws exception to app for getRecord(), insertRec(), appendRec(),
;;      nextrec() errors other than 'usual' ones (EOF, locked by other
;;      user, etc.)
;;
*/
class TBCursor
  : public BCursor
{
  public:

    // Constructor. Make a BCursor object without opening any Paradox tables.

                    TBCursor();

    // Constructor; make a BCursor object and open the specified Paradox table.

                    TBCursor(TBDatabase *dbArg,
                              const char *tableName,
                              int indexID = 0,
                              BOOL saveEveryChange = FALSE,
                              DBIOpenMode openMode = dbiREADWRITE,
                              DBIShareMode shareMode = dbiOPENSHARED,
                              XLTMode xltMode = xltFIELD );

    // Constructor; make a BCursor object and open the specified table of the
    // specified type

                    TBCursor(TBDatabase *dbArg,
                              const char *tableName,
                              const char *tableType,
                              int indexID,
                              DBIOpenMode openMode = dbiREADWRITE,
                              DBIShareMode shareMode = dbiOPENSHARED,
                              XLTMode xltMode = xltFIELD);

    // Constructor; make a BCursor object and open the specified table of the
    // specified type. Use the indexName to identify the index. Note that for
    // dBASE tables, the indexName is used as the indexTagName and the
    // <tablename>.MDX is used as the indexname (idnetifying the production
    // index).

                    TBCursor(TBDatabase *dbArg,
                              const char *tableName,
                              const char *indexName,
                              DBIOpenMode openMode = dbiREADWRITE,
                              DBIShareMode shareMode = dbiOPENSHARED,
                              XLTMode xltMode = xltFIELD);

    // Constructor; make a BCursor object and open the specified table of the
    // specified type. Use the indexName and indexTagName to identify the
    // index. Needed for dBASE tables

                    TBCursor(TBDatabase *dbArg,
                              const char *tableName,
                              const char *indexName,
                              const char *indexTagName,
                              DBIOpenMode openMode = dbiREADWRITE,
                              DBIShareMode shareMode = dbiOPENSHARED,
                              XLTMode xltMode = xltFIELD);

    // Destructor; closes the engine if it's open.

    virtual         ~TBCursor();

    Retcode         appendRec( TBRecord *rec );
    Retcode         appendRec(const void far *buffer, int size);

    Retcode         attach( TBDatabase *dbArg, TABLEHANDLE tableHandle );

    Retcode         clearRec();
    Retcode         clearRec( TBRecord* rec );

    Retcode         getNextRecord( TBRecord *rec );
    Retcode         getNextRecord();

    Retcode         getPreviousRecord( TBRecord *rec );
    Retcode         getPreviousRecord();

    Retcode         getRecord( TBRecord *rec );
    Retcode         getRecord();
    Retcode         getRecord( void far *buffer, int size );

    Retcode         insertRec();
    Retcode         insertRec( TBRecord *rec );
    Retcode         insertRec( const void far *buffer, int size );

    Retcode         open(TBDatabase *dbArg,
                         const char *tableName,
                         int indexID = 0,
                         BOOL saveEveryChange = FALSE,
                         DBIOpenMode openMode = dbiREADWRITE,
                         DBIShareMode shareMode = dbiOPENSHARED,
                         XLTMode xltMode = xltFIELD);
    Retcode         open(TBDatabase *dbArg,
                         const char *tableName,
                         const char *tableType,
                         int indexID,
                         DBIOpenMode openMode = dbiREADWRITE,
                         DBIShareMode shareMode = dbiOPENSHARED,
                         XLTMode xltMode = xltFIELD);
    Retcode         open(TBDatabase *dbArg,
                         const char *tableName,
                         const char *indexName,
                         DBIOpenMode openMode = dbiREADWRITE,
                         DBIShareMode shareMode = dbiOPENSHARED,
                         XLTMode xltMode = xltFIELD);
    Retcode         open(TBDatabase *dbArg,
                         const char *tableName,
                         const char *indexName,
                         const char *indexTagName,
                         DBIOpenMode openMode = dbiREADWRITE,
                         DBIShareMode shareMode = dbiOPENSHARED,
                         XLTMode xltMode = xltFIELD);

    Retcode         searchNonIndex( const char* psFldName, const char* psFldvalExpr, TBoolean bLikePattern = bFalse );

    Retcode         updateRec();
    Retcode         updateRec( TBRecord *rec );
    Retcode         updateRec( const void far *buffer, int size );

  protected:
    Retcode         appendRec(BRecord *rec);
    Retcode         attach(BDatabase *dbArg, TABLEHANDLE tableHandle);
    Retcode         checkExc();
    Retcode         clearRec( BRecord* rec );
    Retcode         getNextRecord( BRecord *rec );
    Retcode         getPreviousRecord( BRecord *rec );
    Retcode         getRecord( BRecord *rec );
    Retcode         insertRec( BRecord *rec );
    Retcode         open(BDatabase *dbArg,
                         const char *tableName,
                         int indexID = 0,
                         BOOL saveEveryChange = FALSE,
                         DBIOpenMode openMode = dbiREADWRITE,
                         DBIShareMode shareMode = dbiOPENSHARED,
                         XLTMode xltMode = xltFIELD);
    Retcode         open(BDatabase *dbArg,
                         const char *tableName,
                         const char *tableType,
                         int indexID,
                         DBIOpenMode openMode = dbiREADWRITE,
                         DBIShareMode shareMode = dbiOPENSHARED,
                         XLTMode xltMode = xltFIELD);
    Retcode         open(BDatabase *dbArg,
                         const char *tableName,
                         const char *indexName,
                         DBIOpenMode openMode = dbiREADWRITE,
                         DBIShareMode shareMode = dbiOPENSHARED,
                         XLTMode xltMode = xltFIELD);
    Retcode         open(BDatabase *dbArg,
                         const char *tableName,
                         const char *indexName,
                         const char *indexTagName,
                         DBIOpenMode openMode = dbiREADWRITE,
                         DBIShareMode shareMode = dbiOPENSHARED,
                         XLTMode xltMode = xltFIELD);
    void            throwExc( const char* fnArg,
                              const char* errArg = NULL,
                              const char* infoArg = NULL );
    Retcode         updateRec( BRecord *rec );

    //
#if 0//950515
    TBDatabase*     ptDb_;
#else
    TBDatabase*     db;
#endif//950515

  private:
    friend class TBDatabase;
    friend class TBRecord;

};

#endif//I_TBCURSOR

