/*
;;tbdataba.hpp
;;
;;  TBDatabase class, inherited from BDatabase
;;
*/

#ifdef    I_TBDATABASE
#else
#define   I_TBDATABASE

#include  <kdbf.h>
#include  <tbengine.hpp>

/*
;;class
;;TSortTable
;;
;;  Struct to store sort-fields specifier for sortTable() function
*/
class
TSortTable
{
public:
         TSortTable();
  //
  UINT16 ahSortFlds_[ DBIMAXFLDSINKEY ];
  UINT16 nSortFlds_;
};

/*
;;class
;;TSortTableFldName
;;
;;  Class to store sort-fields specifier for sortTable() function
;;  by field name
;;
*/
class
TSortTableFldName
{
public:
              TSortTableFldName();
              ~TSortTableFldName();
  //
  TSortTable* ptSortTable_;
  char*       psFldName_[ DBIMAXFLDSINKEY ];
};

class TBCursor;

/*
;;class
;;TBDatabase
;;  : public BDatabase
;;
;;  This class adds the following functionality beyond BDatabase:
;;    -New function CreateTableBorrow() creates an empty table, borrowing
;;      structure from a table in any open database. Typically used to
;;      create tables for local processing.
;;
*/
class
TBDatabase
  : public BDatabase
{
  public:
            TBDatabase( TBEngine *eng,
                       DBIOpenMode openMode = dbiREADWRITE,
                       DBIShareMode shareMode = dbiOPENSHARED);

            TBDatabase( TBEngine *eng,
                       const char *BDDriver,
                       const char *BDAlias = "",
                       const char *BDPassword = "",
                       DBIOpenMode openMode = dbiREADWRITE,
                       DBIShareMode shareMode = dbiOPENSHARED);

            TBDatabase( TBEngine *eng,
                       const char *BDDriver,
                       const char *BDAlias,
                       const char *BDPassword,
                       const char *BDUserName, //This is the new (SQL) parameter
                       DBIOpenMode openMode = dbiREADWRITE,
                       DBIShareMode shareMode = dbiOPENSHARED);

    virtual ~TBDatabase();

    Retcode appendTable(const char *srcTableName,
                        const char *destTableName,
                        const char *destType = NULL,
                        const TBDatabase &db = NULL,
                        eBATMode xBatMode = batAPPEND );

    Retcode appendTable(TBCursor* ptCsrSource,
                        const char *destTableName,
                        const char *destType = NULL,
                        const TBDatabase &db = NULL,
                        eBATMode xBatMode = batAPPEND );

    Retcode appendTable(const char* srcTableName,
                        TBCursor* ptCsrDest,
                        eBATMode xBatMode = batAPPEND );

    Retcode appendTable(TBCursor* ptCsrSrc,
                        TBCursor* ptCsrDest,
                        eBATMode xBatMode = batAPPEND );

    Retcode beginTransaction(hDBIXact &tranHandle, eXILType tranType);
    Retcode createTable(const char *tableName,
                        int numFields, const FieldDesc *desc,
                        ValidityDesc *valDesc = NULL,
                        RefDesc *refDesc = NULL);

    Retcode createTableBorrow( const char* psTblName, const char* psBorrowFrom,
                               TBDatabase* ptDbFrom = NULL );

    Retcode endTransaction( eXEnd tranType );

    Retcode query(const char *qryString, TBCursor &answer,
                  DBIQryLang qryLang = qrylangSQL);

    Retcode queryNoAnswer( const char *qryString, 
                           DBIQryLang qryLang = qrylangSQL);

    Retcode sortTable( const char* psTblname, TSortTable& rtSortTable );
    Retcode sortTable( const char* psTblname, TSortTableFldName& rtSortTableFldName );

  protected:
    Retcode query(const char *qryString, BCursor &answer,
                  DBIQryLang qryLang = qrylangSQL);

    void        throwExc( const char* fnArg,
                          const char* errArg = NULL,
                          const char* infoArg = NULL,
                          const char* aliasArg = NULL );
#if 0//950515
    TBEngine* ptEngine_;
#else
    TBEngine* engine;
#endif//950515

  private:
    friend class BStatement;
    friend class TBEngine;
    friend class TBCursor;
    friend class TBRecord;
    friend class TQuery;
};

#endif//I_TBDATABASE

