/*
;;tbrecord.hpp
;;
;;  TBRecord class, inherited from BRecord
;;
*/

#ifdef    I_TBRECORD
#else
#define   I_TBRECORD

#include  <kdbf.h>

#include  <tbcursor.hpp>

/*
;;class TBRecord
;;  : public BRecord
;;
;;  This class adds the following functionality beyond BRecord:
;;    -Throws exception to app for getField(), putField(), et al.
;;
*/
class TBRecord
  : public BRecord
{
  public:
                    TBRecord(TBCursor *cursor = 0);

  // Destructor; will close the record buffer associated with the record.

    virtual         ~TBRecord();

    virtual Retcode attach( BCursor *curArg );
    virtual Retcode attach( TBCursor *curArg );

    Retcode         closeBlob(FIELDNUMBER fldnbr,
                              BOOL bAccept = TRUE);
    Retcode         closeBlob(const char* psFldName,
                              BOOL bAccept = TRUE);

    Retcode         copyToSimilar( TBRecord* ptRecDest );

    Retcode         dropBlob(FIELDNUMBER fldnbr);
    Retcode         dropBlob(const char* psFldName);

    Retcode         getBlob(FIELDNUMBER fldnbr,
                            unsigned int nSize,
                            long nOffset,
                            void far *pvBuffer);
    Retcode         getBlob(const char* psFldName,
                            unsigned int nSize,
                            long nOffset,
                            void far *pvBuffer);

    Retcode         getBlobHeader(FIELDNUMBER fldnbr,
                                  int nSize,
                                  void* pvBuffer,
                                  int& rnBytesRead);
    Retcode         getBlobHeader(const char* psFldName,
                                  int nSize,
                                  void* pvBuffer,
                                  int& rnBytesRead);

    unsigned long   getBlobSize(FIELDNUMBER fldnbr);
    unsigned long   getBlobSize(const char* psFldName);

    Retcode         getField(FIELDNUMBER fldnbr, char *buf,
                             int bufLen );
    Retcode         getField(FIELDNUMBER fldnbr, void *buf,
                             int bufLen );
    Retcode         getField(FIELDNUMBER fldnbr, double& val );
    Retcode         getField(FIELDNUMBER fldnbr, INT16& val );
    Retcode         getField(FIELDNUMBER fldnbr, INT32& val );
    Retcode         getField(FIELDNUMBER fldnbr, UINT16& val );
    Retcode         getField(FIELDNUMBER fldnbr, UINT32& val );
    Retcode         getField(FIELDNUMBER fldnbr, FMTBcd& val );
    Retcode         getField(FIELDNUMBER fldnbr, BDate& val );
    Retcode         getField(FIELDNUMBER fldnbr, BTime& val );
    Retcode         getField(FIELDNUMBER fldnbr, BTimeStamp& val );
    Retcode         getField(FIELDNUMBER fldnbr, char *buf,
                             int bufLen, BOOL& fNull);
    Retcode         getField(FIELDNUMBER fldnbr, void *buf,
                             int bufLen, BOOL& fNull);
    Retcode         getField(FIELDNUMBER fldnbr, double& val, BOOL& fNull);
    Retcode         getField(FIELDNUMBER fldnbr, INT16& val, BOOL& fNull);
    Retcode         getField(FIELDNUMBER fldnbr, INT32& val, BOOL& fNull);
    Retcode         getField(FIELDNUMBER fldnbr, UINT16& val, BOOL& fNull);
    Retcode         getField(FIELDNUMBER fldnbr, UINT32& val, BOOL& fNull);
    Retcode         getField(FIELDNUMBER fldnbr, FMTBcd& val, BOOL& fNull);
    Retcode         getField(FIELDNUMBER fldnbr, BDate& val, BOOL& fNull);
    Retcode         getField(FIELDNUMBER fldnbr, BTime& val, BOOL& fNull);
    Retcode         getField(FIELDNUMBER fldnbr, BTimeStamp& val, BOOL& fNull);
    Retcode         getField(char *fldName, char *buf, int bufLen,
                               BOOL& fNull);
    Retcode         getField(char *fldName, void *buf, int bufLen,
                               BOOL& fNull);
    Retcode         getField(char *fldName, double& val, BOOL& fNull);
    Retcode         getField(char *fldName, INT16& val, BOOL& fNull);
    Retcode         getField(char *fldName, INT32& val, BOOL& fNull);
    Retcode         getField(char *fldName, UINT16& val, BOOL& fNull);
    Retcode         getField(char *fldName, UINT32& val, BOOL& fNull);
    Retcode         getField(char *fldName, FMTBcd& val, BOOL& fNull);
    Retcode         getField(char *fldName, BDate& val, BOOL& fNull);
    Retcode         getField(char *fldName, BTime& val, BOOL& fNull);
    Retcode         getField(char *fldName, BTimeStamp& val, BOOL& fNull);
    Retcode         getField(const char *fldName, char *buf, int bufLen );
    Retcode         getField(const char *fldName, void *buf, int bufLen );
    Retcode         getField(const char *fldName, double& val );
    Retcode         getField(const char *fldName, INT16& val );
    Retcode         getField(const char *fldName, INT32& val );
    Retcode         getField(const char *fldName, UINT16& val );
    Retcode         getField(const char *fldName, UINT32& val );
    Retcode         getField(const char *fldName, FMTBcd& val );
    Retcode         getField(const char *fldName, BDate& val );
    Retcode         getField(const char *fldName, BTime& val );
    Retcode         getField(const char *fldName, BTimeStamp& val );

    BOOL            isNull(FIELDNUMBER fldnbr);
    BOOL            isNull(const char *fldname);

    Retcode         openBlobRead(FIELDNUMBER fldnbr,
                                  BOOL usePrivateCopy = FALSE);
    Retcode         openBlobRead(const char* psFldName,
                                  BOOL usePrivateCopy = FALSE);

    Retcode         openBlobWrite(FIELDNUMBER fldnbr,
                                  long nSize,
                                  BOOL bCopyOld);
    Retcode         openBlobWrite(const char* psFldName,
                                  long nSize,
                                  BOOL bCopyOld);

    Retcode         putBlob(FIELDNUMBER fldnbr,
                            unsigned long nSize,
                            unsigned long nOffset,
                            void far *pvBuffer);
    Retcode         putBlob(const char* psFldName,
                            unsigned long nSize,
                            unsigned long nOffset,
                            void far *pvBuffer);

    Retcode         putField(FIELDNUMBER fldnbr, const char *buf);
    Retcode         putField(FIELDNUMBER fldnbr, const void *buf);
    Retcode         putField(FIELDNUMBER fldnbr, double val);
    Retcode         putField(FIELDNUMBER fldnbr, INT16 val);
    Retcode         putField(FIELDNUMBER fldnbr, INT32 val);
    Retcode         putField(FIELDNUMBER fldnbr, UINT16 val);
    Retcode         putField(FIELDNUMBER fldnbr, UINT32 val);
    Retcode         putField(FIELDNUMBER fldnbr, FMTBcd val);
    Retcode         putField(FIELDNUMBER fldnbr, const BDate& val);
    Retcode         putField(FIELDNUMBER fldnbr, const BTime& val);
    Retcode         putField(FIELDNUMBER fldnbr, const BTimeStamp& val);
    Retcode         putField(char *fldName, const char *buf);
    Retcode         putField(char *fldName, const void *buf);
    Retcode         putField(char *fldName, double val);
    Retcode         putField(char *fldName, INT16 val);
    Retcode         putField(char *fldName, INT32 val);
    Retcode         putField(char *fldName, UINT16 val);
    Retcode         putField(char *fldName, UINT32 val);
    Retcode         putField(char *fldName, FMTBcd val);
    Retcode         putField(char *fldName, const BDate& val);
    Retcode         putField(char *fldName, const BTime& val);
    Retcode         putField(char *fldName, const BTimeStamp& val);

    Retcode         setNull(FIELDNUMBER fldnbr);
    Retcode         setNull(const char *fldName);

  protected:
    Retcode         checkExc();
    void            throwExc( const char* fnArg,
                              const char* errArg = NULL,
                              const char* infoArg = NULL,
                              const char* fldArg = NULL );
    //
    TBCursor*       csr;
  private:
    friend class TBCursor;
 };

#endif//I_TBRECORD

