/*
;;tbcursor.cpp
;;
*/

#pragma   hdrstop

#include  <tdbf.hpp>

/*
;;TBCursor::TBCursor()
;;  : BCursor()
;;
*/
TBCursor::TBCursor()
  : BCursor()
{
}

/*
;;TBCursor::TBCursor( TBDatabase *dbArg,
;;                  const char *tableName,
;;                  int indexID = 0,
;;                  BOOL saveEveryChange = FALSE,
;;                  DBIOpenMode openMode = dbiREADWRITE,
;;                  DBIShareMode shareMode = dbiOPENSHARED,
;;                  XLTMode xltMode = xltFIELD)
;;  : TBCursor( dbArg, tableName, indexID, saveEveryChange, openMode, shareMode,
;;              xltMode )
;;
;;  Constructor; make a TBCursor object and open the specified Paradox table.
;;
;;  Throws a (Retcode) exception on any error
;;
*/
TBCursor::TBCursor( TBDatabase *dbArg,
                  const char *tableName,
                  int indexID,
                  BOOL saveEveryChange,
                  DBIOpenMode openMode,
                  DBIShareMode shareMode,
                  XLTMode xltMode )
  : BCursor( dbArg, tableName, indexID, saveEveryChange, openMode, shareMode,
              xltMode ),
#if 0//950515
    ptDb_( db )
#else
    db( dbArg )
#endif//950515
{
#if 0//950517
  DELETE_SAFE( genericRec );
#endif//950517
  //Checking for any error. ! isOpen should always have a (DBIERR_NONE != lastError )
  //  Should get a throw ( xalloc ) on memory error.
  if ( DBIERR_NONE != lastError )
  {
#if 0//950520
    throw ( lastError );
#else
    TUtil::ThrowExc( "TBCursor()", "BCursor constructor", NULL, "TBCursor",
                     lastError, db->Alias, tableName );
#endif//950520
  }
}

/*
;;TBCursor::TBCursor( TBDatabase *dbArg,
;;                  const char *tableName,
;;                  const char *tableType,
;;                  int indexID,
;;                  DBIOpenMode openMode = dbiREADWRITE,
;;                  DBIShareMode shareMode = dbiOPENSHARED,
;;                  XLTMode xltMode = xltFIELD)
;;  : BCursor( dbArg, tableName, tableType, indexID, openMode, shareMode, xltMode )
;;
;;  Constructor; make a TBCursor object and open the specified table of the
;;  specified type
;;
*/
TBCursor::TBCursor( TBDatabase *dbArg,
                  const char *tableName,
                  const char *tableType,
                  int indexID,
                  DBIOpenMode openMode,
                  DBIShareMode shareMode,
                  XLTMode xltMode )
  : BCursor( dbArg, tableName, tableType, indexID, openMode, shareMode, xltMode ),
#if 0//950515
    ptDb_( db )
#else
    db( dbArg )
#endif//950515
{
#if 0//950517
  DELETE_SAFE( genericRec );
#endif//950517
  //Checking for any error. ! isOpen should always have a (DBIERR_NONE != lastError )
  //  Should get a throw ( xalloc ) on memory error.
  if ( DBIERR_NONE != lastError )
  {
#if 0//950520
    throw ( lastError );
#else
    TUtil::ThrowExc( "TBCursor()", "BCursor constructor", NULL, "TBCursor",
                     lastError, db->Alias, tableName );
#endif//950520
  }
}

/*
;;TBCursor::TBCursor( TBDatabase *dbArg,
;;                  const char *tableName,
;;                  const char *indexName,
;;                  DBIOpenMode openMode = dbiREADWRITE,
;;                  DBIShareMode shareMode = dbiOPENSHARED,
;;                  XLTMode xltMode = xltFIELD )
;;  : BCursor()
;;
;;  Constructor; make a TBCursor object and open the specified table of the
;;  specified type. Use the indexName to identify the index. Note that for
;;  dBASE tables, the indexName is used as the indexTagName and the
;;  <tablename>.MDX is used as the indexname (idnetifying the production
;;  index).
;;
;;  941123
;;    -Added XPRIM reserved suffix support
;;
*/
TBCursor::TBCursor( TBDatabase *dbArg,
                  const char *tableName,
                  const char *indexName,
                  DBIOpenMode openMode,
                  DBIShareMode shareMode,
                  XLTMode xltMode )
  //: BCursor( dbArg, tableName, indexName, openMode, shareMode, xltMode )
  : BCursor(),
#if 0//950515
    ptDb_( db )
#else
    db( dbArg )
#endif//950515
{
#if 0//950517
  DELETE_SAFE( genericRec );
#endif//950517
#if 0
  if ( 0 < strstr( indexName, "XPRIM" ) ) //Assume upper case!
  {
    if ( 0 == strcmp( szPARADOX, db->getDriver() ) ) //? Paradox
    {
      open( db, tableName, 0, openMode, shareMode, xltMode );
    }
    else //Non-Paradox, assume SQL
    {
      open( db, tableName, indexName, openMode, shareMode, xltMode );
    }
  }
  else
  {
    open( db, tableName, indexName, openMode, shareMode, xltMode );
  }
  //Checking for any error. ! isOpen should always have a (DBIERR_NONE != lastError )
  //  Should get a throw ( xalloc ) on memory error.
  if ( DBIERR_NONE != lastError )
  {
    throw ( lastError );
  }
#else
  if ( DBIERR_NONE != lastError )
  {
    throwExc( "TBCursor()", "BCursor constructor" );
  }
  try
  {
    open( db, tableName, indexName, openMode, shareMode, xltMode );
  }
  catch (TTdbfExc )
  {
    TUtil::ThrowExc( "TBCursor()", "BCursor constructor", NULL, "TBCursor",
                     lastError, db->Alias, tableName );
  }
  if ( DBIERR_NONE != lastError )
  {
    throwExc( "TBCursor()" );
  }
#endif//0
}

/*
;;TBCursor::TBCursor( TBDatabase *dbArg,
;;                  const char *tableName,
;;                  const char *indexName,
;;                  const char *indexTagName,
;;                  DBIOpenMode openMode = dbiREADWRITE,
;;                  DBIShareMode shareMode = dbiOPENSHARED,
;;                  XLTMode xltMode = xltFIELD)
;;  : BCursor( dbArg, tableName, indexName, indexTagName, openMode, shareMode,
;;              xltMode )
;;
;;  Constructor; make a TBCursor object and open the specified table of the
;;  specified type. Use the indexName and indexTagName to identify the
;;  index. Needed for dBASE tables
;;
*/
TBCursor::TBCursor( TBDatabase *dbArg,
                  const char *tableName,
                  const char *indexName,
                  const char *indexTagName,
                  DBIOpenMode openMode,
                  DBIShareMode shareMode,
                  XLTMode xltMode )
  : BCursor( dbArg, tableName, indexName, indexTagName, openMode, shareMode,
              xltMode ),
#if 0//950515
    ptDb_( db )
#else
    db( dbArg )
#endif//950515
{
#if 0//950517
  DELETE_SAFE( genericRec );
#endif//950517
  //Checking for any error. ! isOpen should always have a (DBIERR_NONE != lastError )
  //  Should get a throw ( xalloc ) on memory error.
  if ( DBIERR_NONE != lastError )
  {
#if 0//950520
    throw ( lastError );
#else
    TUtil::ThrowExc( "TBCursor()", "BCursor constructor", NULL, "TBCursor",
                     lastError, db->Alias, tableName );
#endif//950520
  }
}

/*
;;protected:
;;Retcode
;;TBCursor::attach(BDatabase *dbArg, TABLEHANDLE tableHandle)
;;
*/
Retcode
TBCursor::attach(BDatabase *dbArg, TABLEHANDLE tableHandle)
{
  return ( BCursor::attach( dbArg, tableHandle ) );
}

/*
;;Retcode
;;TBCursor::attach( TBDatabase *dbArg, TABLEHANDLE tableHandle )
;;
*/
Retcode
TBCursor::attach( TBDatabase *dbArg, TABLEHANDLE tableHandle )
{
  db = dbArg;
  BCursor::attach( db, tableHandle );
  if ( DBIERR_NONE != lastError )
  {
    throwExc( "attach()" );
  }
  return ( lastError );
}

/*
;;virtual
;;TBCursor::~TBCursor()
;;
;;  Destructor; closes the engine if it's open.
;;
*/
TBCursor::~TBCursor()
{
}

/*
;;protected:
;;Retcode
;;TBCursor::appendRec(BRecord *rec)
;;
*/
Retcode
TBCursor::appendRec(BRecord *rec)
{
  return ( BCursor::appendRec( rec ) );
}

/*
;;Retcode
;;TBCursor::appendRec( TBRecord *rec)
;;
;;  Throws a (Retcode) exception on anything but:
;;    DBIERR_NONE
;;    DBIERR_BOF
;;    DBIERR_EOF
;;    DBIERR_FILELOCKED     (likely for this funct.)
;;    DBIERR_KEYVIOL        (likely for this funct.)
;;    DBIERR_RECLOCKFAILED  (likely for this funct.)
;;    PXERR_INVCURRRECORD
;;
;;    (otherwise returns one of the above)
;;
*/
Retcode
TBCursor::appendRec( TBRecord *rec )
{
  BCursor::appendRec( rec );
  switch ( lastError )
  {
    case DBIERR_BLOBNOTOPENED : //This BLOB was never opened
    //case PXERR_BLOBNOTOPEN : //951114
      lastError = DBIERR_NONE;
      break;
    case DBIERR_NONE :
    default :
      break;
  }
#if 0//950520
  return ( checkExc() );
#else
  if ( DBIERR_NONE != checkExc() )
  {
    throwExc( "appendRec()" );
  }
  return ( lastError );
#endif//950520
}

/*
;;Retcode
;;TBCursor::appendRec(const void far *buffer, int size)
;;
;;  Throws a (Retcode) exception on anything but:
;;    DBIERR_NONE
;;    DBIERR_BOF
;;    DBIERR_EOF
;;    DBIERR_FILELOCKED     (likely for this funct.)
;;    DBIERR_KEYVIOL        (likely for this funct.)
;;    DBIERR_RECLOCKFAILED  (likely for this funct.)
;;    PXERR_INVCURRRECORD
;;
;;    (otherwise returns one of the above)
;;
*/
Retcode
TBCursor::appendRec(const void far *buffer, int size)
{
  BCursor::appendRec( buffer, size );
#if 0//950520
  return ( checkExc() );
#else
  if ( DBIERR_NONE != checkExc() )
  {
    throwExc( "appendRec()" );
  }
  return ( lastError );
#endif//950520
}

/*
;;protected:
;;Retcode
;;TBCursor::checkExc()
;;
;;  Check if lastError is not a normal one, throw (Retcode) exception to app
;;    if so, otherwise return DBIERR_NONE or 'normal' error
;;
*/
Retcode
TBCursor::checkExc()
{
  switch ( lastError )
  {
    case DBIERR_NONE :
    case DBIERR_BOF :
    case DBIERR_EOF :
    case DBIERR_FILELOCKED :
    case DBIERR_KEYVIOL :
    case DBIERR_RECLOCKFAILED :
    case PXERR_INVCURRRECORD :
      break;
    default :
#if 0//950520
      throw ( lastError );          //Then throw to app
  }
  return ( lastError );           //(Always DBIERR_NONE)
#else
      return ( lastError );       //Will be thrown to app
  }
  return ( DBIERR_NONE );           //(Always DBIERR_NONE)
#endif//950520
}

/*
;;Retcode
;;TBCursor::clearRec()
;;
*/
Retcode
TBCursor::clearRec()
{
  return ( clearRec( genericRec ) );
}

/*
;;protected:
;;Retcode
;;TBCursor::clearRec( BRecord* rec )
;;
*/
Retcode
TBCursor::clearRec( BRecord* rec )
{
  return ( BCursor::clearRec( rec ) );
}

/*
;;Retcode
;;TBCursor::clearRec( TBRecord* rec )
;;
;;  Used to clear any open blobs. This function should be called
;;  if any blobs have been written to a record, but appendRec(),
;;  insertRec() or updateRec() was never called successfully.
;;
;;  For example, if there is a locking conflict, and a decision
;;  is made not to post a record having had blobs written to its
;;  record buffer, clearRec() should be called to free those blob
;;  resources. clearRec() may be called even after the blobs have
;;  already been closed by appendRec(), insertRec() or updateRec().
;;
;;  In summary, whenever a blob is written to a BRecord, it MUST
;;  be followed by a successfull call to one of the following BCursor
;;  functions, or BLOB resources will not be freed:
;;    appendRec()
;;    clearRec()
;;    insertRec()
;;    updateRec()
;;
;;  Throws a (Retcode) exception on any error.
;;
*/
Retcode
TBCursor::clearRec( TBRecord* rec )
{
#if 0//Now in KDBF
  FIELDNUMBER n = rec->getFieldCount();
  FIELDNUMBER h;
  FieldDesc   tFldDesc;
  //
  for ( h = 1; n >= h; ++h )
  {
    lastError = rec->getFieldDesc(h, tFldDesc) ;
    if ( DBIERR_NONE != lastError )
    {
      throw ( lastError );
    }
    if ( fldBlob != tFldDesc.fldType )
    {
      continue;
    }
    lastError = rec->closeBlob( h, FALSE );
    switch ( lastError )
    {
      case DBIERR_NONE :
        break;
      case DBIERR_BLOBNOTOPENED : //appendRec(), insertRec() or updateRec()
                                  //  must have already been called, or
                                  //  this BLOB was never opened
      case PXERR_BLOBNOTOPEN :
        lastError = DBIERR_NONE;
        break;
      default :
        throw ( lastError );
    }
  }
  return ( lastError );
#else
  BCursor::clearRec( rec );
  switch ( lastError )
  {
    case DBIERR_NONE :
      break;
    case DBIERR_BLOBNOTOPENED : //appendRec(), insertRec() or updateRec()
                                //  must have already been called, or
                                //  this BLOB was never opened
    //case PXERR_BLOBNOTOPEN : //951114
      lastError = DBIERR_NONE;
      break;
    default :
#if 0//950520
      throw ( lastError );
#else
      throwExc( "clearRec()" );
#endif//950520
  }
  return ( lastError );
#endif//0
}

/*
;;protected:
;;Retcode
;;TBCursor::getRecord(BRecord *rec)
;;
*/
Retcode
TBCursor::getRecord(BRecord *rec)
{
  return ( BCursor::getRecord( rec ) );
}

/*
;;Retcode
;;TBCursor::getRecord(TBRecord *rec)
;;
;;  Throws a (Retcode) exception on anything but:
;;    DBIERR_NONE
;;    DBIERR_BOF
;;    DBIERR_EOF            (likely for this funct.)
;;    DBIERR_FILELOCKED
;;    DBIERR_KEYVIOL
;;    DBIERR_RECLOCKFAILED
;;    PXERR_INVCURRRECORD   (likely for this funct.)
;;
;;    (otherwise returns one of the above)
;;
*/
Retcode
TBCursor::getRecord(TBRecord *rec)
{
  BCursor::getRecord( rec );
#if 0//950520
  return ( checkExc() );
#else
  if ( DBIERR_NONE != checkExc() )
  {
    throwExc( "getRecord()" );
  }
  return ( lastError );
#endif//950520
}

/*
;;Retcode
;;TBCursor::getRecord()
;;
;;  Throws a (Retcode) exception on anything but:
;;    DBIERR_NONE
;;    DBIERR_BOF
;;    DBIERR_EOF            (likely for this funct.)
;;    DBIERR_FILELOCKED
;;    DBIERR_KEYVIOL
;;    DBIERR_RECLOCKFAILED
;;    PXERR_INVCURRRECORD   (likely for this funct.)
;;
;;    (otherwise returns one of the above)
;;
*/
Retcode
TBCursor::getRecord()
{
  BCursor::getRecord();
#if 0//950520
  return ( checkExc() );
#else
  if ( DBIERR_NONE != checkExc() )
  {
    throwExc( "getRecord()" );
  }
  return ( lastError );
#endif//950520
}

/*
;;Retcode
;;TBCursor::getRecord(void far *buffer, int size)
;;
;;  Throws a (Retcode) exception on anything but:
;;    DBIERR_NONE
;;    DBIERR_BOF
;;    DBIERR_EOF            (likely for this funct.)
;;    DBIERR_FILELOCKED
;;    DBIERR_KEYVIOL
;;    DBIERR_RECLOCKFAILED
;;    PXERR_INVCURRRECORD   (likely for this funct.)
;;
;;    (otherwise returns one of the above)
;;
*/
Retcode
TBCursor::getRecord(void far *buffer, int size)
{
  BCursor::getRecord( buffer, size );
#if 0//950520
  return ( checkExc() );
#else
  if ( DBIERR_NONE != checkExc() )
  {
    throwExc( "getRecord()" );
  }
  return ( lastError );
#endif//950520
}

/*
;;protected:
;;Retcode
;;TBCursor::getNextRecord(BRecord *rec)
;;
*/
Retcode
TBCursor::getNextRecord(BRecord *rec)
{
  return ( BCursor::getNextRecord( rec ) );
}

/*
;;Retcode
;;TBCursor::getNextRecord(TBRecord *rec)
;;
;;  Throws a (Retcode) exception on anything but:
;;    DBIERR_NONE
;;    DBIERR_BOF
;;    DBIERR_EOF            (likely for this funct.)
;;    DBIERR_FILELOCKED
;;    DBIERR_KEYVIOL
;;    DBIERR_RECLOCKFAILED
;;    PXERR_INVCURRRECORD   (likely for this funct.)
;;
;;    (otherwise returns one of the above)
;;
*/
Retcode
TBCursor::getNextRecord(TBRecord *rec)
{
  BCursor::getNextRecord( rec );
#if 0//950520
  return ( checkExc() );
#else
  if ( DBIERR_NONE != checkExc() )
  {
    throwExc( "getNextRecord()" );
  }
  return ( lastError );
#endif//950520
}

/*
;;Retcode
;;TBCursor::getNextRecord()
;;
;;  Throws a (Retcode) exception on anything but:
;;    DBIERR_NONE
;;    DBIERR_BOF
;;    DBIERR_EOF            (likely for this funct.)
;;    DBIERR_FILELOCKED
;;    DBIERR_KEYVIOL
;;    DBIERR_RECLOCKFAILED
;;    PXERR_INVCURRRECORD   (likely for this funct.)
;;
;;    (otherwise returns one of the above)
;;
*/
Retcode
TBCursor::getNextRecord()
{
  BCursor::getNextRecord();
#if 0//950520
  return ( checkExc() );
#else
  if ( DBIERR_NONE != checkExc() )
  {
    throwExc( "getNextRecord()" );
  }
  return ( lastError );
#endif//950520
}

/*
;;protected:
;;Retcode
;;TBCursor::getPreviousRecord(BRecord *rec)
;;
*/
Retcode
TBCursor::getPreviousRecord(BRecord *rec)
{
  return ( BCursor::getPreviousRecord( rec ) );
}

/*
;;Retcode
;;TBCursor::getPreviousRecord( TBRecord *rec )
;;
;;  Throws a (Retcode) exception on anything but:
;;    DBIERR_NONE
;;    DBIERR_BOF            (likely for this funct.)
;;    DBIERR_EOF
;;    DBIERR_FILELOCKED
;;    DBIERR_KEYVIOL
;;    DBIERR_RECLOCKFAILED
;;    PXERR_INVCURRRECORD   (likely for this funct.)
;;
;;    (otherwise returns one of the above)
;;
*/
Retcode
TBCursor::getPreviousRecord( TBRecord *rec )
{
  BCursor::getPreviousRecord( rec );
#if 0//950520
  return ( checkExc() );
#else
  if ( DBIERR_NONE != checkExc() )
  {
    throwExc( "getPreviousRecord()" );
  }
  return ( lastError );
#endif//950520
}

/*
;;Retcode
;;TBCursor::getPreviousRecord()
;;
;;  Throws a (Retcode) exception on anything but:
;;    DBIERR_NONE
;;    DBIERR_BOF            (likely for this funct.)
;;    DBIERR_EOF
;;    DBIERR_FILELOCKED
;;    DBIERR_KEYVIOL
;;    DBIERR_RECLOCKFAILED
;;    PXERR_INVCURRRECORD   (likely for this funct.)
;;
;;    (otherwise returns one of the above)
;;
*/
Retcode
TBCursor::getPreviousRecord()
{
  BCursor::getPreviousRecord();
#if 0//950520
  return ( checkExc() );
#else
  if ( DBIERR_NONE != checkExc() )
  {
    throwExc( "getPreviousRecord()" );
  }
  return ( lastError );
#endif//950520
}

/*
;;Retcode
;;TBCursor::insertRec()
;;
;;  Throws a (Retcode) exception on anything but:
;;    DBIERR_NONE
;;    DBIERR_BOF
;;    DBIERR_EOF
;;    DBIERR_FILELOCKED     (likely for this funct.)
;;    DBIERR_KEYVIOL        (likely for this funct.)
;;    DBIERR_RECLOCKFAILED  (likely for this funct.)
;;    PXERR_INVCURRRECORD
;;
;;    (otherwise returns one of the above)
;;
*/
Retcode
TBCursor::insertRec()
{
  BCursor::insertRec();
#if 0//950520
  return ( checkExc() );
#else
  if ( DBIERR_NONE != checkExc() )
  {
    throwExc( "insertRec()" );
  }
  return ( lastError );
#endif//950520
}

/*
;;protected:
;;Retcode
;;TBCursor::insertRec(BRecord *rec)
;;
*/
Retcode
TBCursor::insertRec(BRecord *rec)
{
  return ( BCursor::insertRec( rec ) );
}

/*
;;Retcode
;;TBCursor::insertRec(TBRecord *rec)
;;
;;  Throws a (Retcode) exception on anything but:
;;    DBIERR_NONE
;;    DBIERR_BOF
;;    DBIERR_EOF
;;    DBIERR_FILELOCKED     (likely for this funct.)
;;    DBIERR_KEYVIOL        (likely for this funct.)
;;    DBIERR_RECLOCKFAILED  (likely for this funct.)
;;    PXERR_INVCURRRECORD
;;
;;    (otherwise returns one of the above)
;;
*/
Retcode
TBCursor::insertRec(TBRecord *rec)
{
  BCursor::insertRec( rec );
  switch ( lastError )
  {
    case DBIERR_BLOBNOTOPENED : //This BLOB was never opened
    //case PXERR_BLOBNOTOPEN : //951114
      lastError = DBIERR_NONE;
      break;
    case DBIERR_NONE :
    default :
      break;
  }
#if 0//950520
  return ( checkExc() );
#else
  if ( DBIERR_NONE != checkExc() )
  {
    throwExc( "insertRec()" );
  }
  return ( lastError );
#endif//950520
}

/*
;;Retcode
;;TBCursor::insertRec(const void far *buffer, int size)
;;
;;  Throws a (Retcode) exception on anything but:
;;    DBIERR_NONE
;;    DBIERR_BOF
;;    DBIERR_EOF
;;    DBIERR_FILELOCKED     (likely for this funct.)
;;    DBIERR_KEYVIOL        (likely for this funct.)
;;    DBIERR_RECLOCKFAILED  (likely for this funct.)
;;    PXERR_INVCURRRECORD
;;
;;    (otherwise returns one of the above)
;;
*/
Retcode
TBCursor::insertRec(const void far *buffer, int size)
{
  BCursor::insertRec( buffer, size );
#if 0//950520
  return ( checkExc() );
#else
  if ( DBIERR_NONE != checkExc() )
  {
    throwExc( "insertRec()" );
  }
  return ( lastError );
#endif//950520
}

/*
;;protected:
;;Retcode
;;TBCursor::open( BDatabase *dbArg,
;;                const char *tableName,
;;                int indexID = 0,
;;                BOOL saveEveryChange = FALSE,
;;                DBIOpenMode openMode = dbiREADWRITE,
;;                DBIShareMode shareMode = dbiOPENSHARED,
;;                XLTMode xltMode = xltFIELD)
;;
;;  Throws a (Retcode) exception on any error
;;
*/
Retcode
TBCursor::open( BDatabase *dbArg,
                const char *tableName,
                int indexID,
                BOOL saveEveryChange,
                DBIOpenMode openMode,
                DBIShareMode shareMode,
                XLTMode xltMode)
{
  return ( BCursor::open( (TBDatabase*) dbArg, tableName, indexID, saveEveryChange, openMode, shareMode, xltMode ) );
}

/*
;;protected:
;;Retcode
;;TBCursor::open( BDatabase *dbArg,
;;                const char *tableName,
;;                const char *tableType,
;;                int indexID,
;;                DBIOpenMode openMode = dbiREADWRITE,
;;                DBIShareMode shareMode = dbiOPENSHARED,
;;                XLTMode xltMode = xltFIELD)
;;
;;  Throws a (Retcode) exception on any error
;;
*/
Retcode
TBCursor::open( BDatabase *dbArg,
                const char *tableName,
                const char *tableType,
                int indexID,
                DBIOpenMode openMode,
                DBIShareMode shareMode,
                XLTMode xltMode )
{
  return ( BCursor::open( (TBDatabase*) dbArg, tableName, tableType, indexID, openMode, shareMode, xltMode ) );
}

/*
;;protected:
;;Retcode
;;TBCursor::open( BDatabase *dbArg,
;;                const char *tableName,
;;                const char *indexName,
;;                DBIOpenMode openMode,
;;                DBIShareMode shareMode,
;;                XLTMode xltMode)
;;
;;  Throws a (Retcode) exception on any error
;;
;;  941123
;;    -Added XPRIM reserved suffix support
;;
*/
Retcode
TBCursor::open( BDatabase *dbArg,
                const char *tableName,
                const char *indexName,
                DBIOpenMode openMode,
                DBIShareMode shareMode,
                XLTMode xltMode )
{
  return ( BCursor::open( (TBDatabase*) dbArg, tableName, indexName, openMode, shareMode, xltMode ) );
}

/*
;;protected:
;;Retcode
;;TBCursor::open( BDatabase *dbArg,
;;                const char *tableName,
;;                const char *indexName,
;;                const char *indexTagName,
;;                DBIOpenMode openMode = dbiREADWRITE,
;;                DBIShareMode shareMode = dbiOPENSHARED,
;;                XLTMode xltMode = xltFIELD)
;;
;;  Throws a (Retcode) exception on any error
;;
*/
Retcode
TBCursor::open( BDatabase *dbArg,
                const char *tableName,
                const char *indexName,
                const char *indexTagName,
                DBIOpenMode openMode,
                DBIShareMode shareMode,
                XLTMode xltMode )
{
  return ( BCursor::open( ( TBDatabase*) dbArg, tableName, indexName, indexTagName, openMode, shareMode, xltMode ) );
}

/*
;;Retcode
;;TBCursor::open( TBDatabase *dbArg,
;;                const char *tableName,
;;                int indexID = 0,
;;                BOOL saveEveryChange = FALSE,
;;                DBIOpenMode openMode = dbiREADWRITE,
;;                DBIShareMode shareMode = dbiOPENSHARED,
;;                XLTMode xltMode = xltFIELD)
;;
;;  Throws a (Retcode) exception on any error
;;
*/
Retcode
TBCursor::open( TBDatabase *dbArg,
                const char *tableName,
                int indexID,
                BOOL saveEveryChange,
                DBIOpenMode openMode,
                DBIShareMode shareMode,
                XLTMode xltMode)
{
  db = dbArg;
  BCursor::open( db, tableName, indexID, saveEveryChange, openMode, shareMode, xltMode );
#if 0//950517
  DELETE_SAFE( genericRec );
#endif//950517
  if ( DBIERR_NONE != lastError )
  {
#if 0//950520
    throw ( lastError );
#else
    TUtil::ThrowExc( "open()", NULL, NULL, "TBCursor",
                     lastError, db->Alias, tableName );
#endif//950520
  }
  return ( lastError );
}

/*
;;Retcode
;;TBCursor::open( TBDatabase *dbArg,
;;                const char *tableName,
;;                const char *tableType,
;;                int indexID,
;;                DBIOpenMode openMode = dbiREADWRITE,
;;                DBIShareMode shareMode = dbiOPENSHARED,
;;                XLTMode xltMode = xltFIELD)
;;
;;  Throws a (Retcode) exception on any error
;;
*/
Retcode
TBCursor::open( TBDatabase *dbArg,
                const char *tableName,
                const char *tableType,
                int indexID,
                DBIOpenMode openMode,
                DBIShareMode shareMode,
                XLTMode xltMode )
{
  db = dbArg;
  BCursor::open( db, tableName, tableType, indexID, openMode, shareMode, xltMode );
#if 0//950517
  DELETE_SAFE( genericRec );
#endif//950517
  if ( DBIERR_NONE != lastError )
  {
#if 0//950520
    throw ( lastError );
#else
    TUtil::ThrowExc( "open()", NULL, NULL, "TBCursor",
                     lastError, db->Alias, tableName );
#endif//950520
  }
  return ( lastError );
}

/*
;;Retcode
;;TBCursor::open( TBDatabase *dbArg,
;;                const char *tableName,
;;                const char *indexName,
;;                DBIOpenMode openMode,
;;                DBIShareMode shareMode,
;;                XLTMode xltMode)
;;
;;  Throws a (Retcode) exception on any error
;;
;;  941123
;;    -Added XPRIM reserved suffix support
;;
*/
Retcode
TBCursor::open( TBDatabase *dbArg,
                const char *tableName,
                const char *indexName,
                DBIOpenMode openMode,
                DBIShareMode shareMode,
                XLTMode xltMode )
{
  db = dbArg;
#if 0//950517
  DELETE_SAFE( genericRec );
#endif//950517
  if ( 0 < strstr( indexName, "XPRIM" ) ) //Assume upper case!
  {
    if ( 0 == strcmp( szPARADOX, db->getDriver() ) ) //? Paradox
    {
      BCursor::open( db, tableName, 0, openMode, shareMode, xltMode );
    }
    else //Non-Paradox, assume SQL
    {
      BCursor::open( db, tableName, indexName, openMode, shareMode, xltMode );
    }
  }
  else
  {
    BCursor::open( db, tableName, indexName, openMode, shareMode, xltMode );
  }
  //Checking for any error. ! isOpen should always have a (DBIERR_NONE != lastError )
  //  Should get a throw ( xalloc ) on memory error.
  if ( DBIERR_NONE != lastError )
  {
#if 0//950520
    throw ( lastError );
#else
    TUtil::ThrowExc( "open()", NULL, NULL, "TBCursor",
                     lastError, db->Alias, tableName );
#endif//950520
  }
  return ( lastError );
}

/*
;;Retcode
;;TBCursor::open( TBDatabase *dbArg,
;;                const char *tableName,
;;                const char *indexName,
;;                const char *indexTagName,
;;                DBIOpenMode openMode = dbiREADWRITE,
;;                DBIShareMode shareMode = dbiOPENSHARED,
;;                XLTMode xltMode = xltFIELD)
;;
;;  Throws a (Retcode) exception on any error
;;
*/
Retcode
TBCursor::open( TBDatabase *dbArg,
                const char *tableName,
                const char *indexName,
                const char *indexTagName,
                DBIOpenMode openMode,
                DBIShareMode shareMode,
                XLTMode xltMode )
{
  db = dbArg;
#if 0//950517
  DELETE_SAFE( genericRec );
#endif//950517
  BCursor::open( db, tableName, indexName, indexTagName, openMode, shareMode, xltMode );
  if ( DBIERR_NONE != lastError )
  {
#if 0//950520
    throw ( lastError );
#else
    TUtil::ThrowExc( "open()", NULL, NULL, "TBCursor",
                     lastError, db->Alias, tableName );
#endif//950520
  }
  return ( lastError );
}

/*
;;Retcode
;;TBCursor::searchNonIndex( const char* psFldName,
;;                          const char* psFldvalExpr,
;;                          TBoolean    bLikePattern = bFalse )
;;
;;  Search on single non-indexed field. Table must have a unique key index.
;;    1.) Runs query on table, returns ERRCODE_RECNOTFOUND if 0 recs result
;;    2.) Otherwise uses searchIndex() to return first found occurence
;;          in this TBCursor
;;
    941202
      o Change so query returns only key fields and search field
      o Need non-indexed search--for now, just say found, don't move cursor
*/
Retcode
TBCursor::searchNonIndex( const char* psFldName, const char* psFldvalExpr,
                          TBoolean bLikePattern )
{
  TBCursor* ptCsr;
  TBRecord* ptRec;
#if 0//950515
  TQuery tQry( ptDb_->ptEngine_, ptDb_ );
#else
  TQuery tQry( db->engine, db );
#endif//950515
  strstream tSql;
  //
  tSql.flush();
  tSql << "select * "
          "from " << tabname << " "
          "where " << psFldName << " ";
  if ( bLikePattern )
  {
    tSql << "like '" << psFldvalExpr << "' ";
  }
  else
  {
    tSql << "= '" << psFldvalExpr << "' ";
  }
  lastError = tQry.Query( &tSql, ptCsr, ptRec );
  if( DBIERR_NONE != lastError )
  {
    return ( lastError );
  }
  if ( 0 == ptCsr->getRecCount() )
  {
    lastError = ERRCODE_RECNOTFOUND;
    return ( lastError );
  }
  lastError = ptCsr->getNextRecord( ptRec );
  if ( DBIERR_NONE != lastError )
  {
    return ( lastError );
  }
  //Have key value to locate
#if 0
  IDXDesc tIdxDesc;
  lastError = DbiGetIndexDesc(tabH, 0, &tIdxDesc);
  if ( DBIERR_NONE != lastError )
  {
    return ( lastError );
  }
  if ( 0 == tIdxDesc.iFldsInKey )
  {
    ;//Need non-indexed search--for now, just say found, don't move cursor
  }
  else
  {
    lastError = searchIndex( ptRec, (PXSearchMode) keySEARCHEQ, tIdxDesc.iFldsInKey );
  }
#endif//0
  return ( lastError );
}

/*
;;protected:
;;void
;;TBCursor::throwExc( const char* fnArg,
;;                    const char* errArg = NULL,
;;                    const char* infoArg = NULL )
;;
*/
void
TBCursor::throwExc( const char* fnArg,
                    const char* errArg,
                    const char* infoArg )
{
  const char* strAlias = NULL;
  const char* strTabname = NULL;
  if ( isOpen ) //Only if isOpen
  {
    strAlias = db->Alias;
    strTabname = tabname;
  }
  TUtil::ThrowExc( fnArg,
                   errArg,
                   infoArg,
                   "TBCursor",
                   lastError,
                   strAlias,
                   strTabname );
}

/*
;;Retcode
;;TBCursor::updateRec()
;;
;;  Throws a (Retcode) exception on anything but:
;;    DBIERR_NONE
;;    DBIERR_BOF
;;    DBIERR_EOF
;;    DBIERR_FILELOCKED     (likely for this funct.)
;;    DBIERR_KEYVIOL        (likely for this funct.)
;;    DBIERR_RECLOCKFAILED  (likely for this funct.)
;;    PXERR_INVCURRRECORD
;;
;;    (otherwise returns one of the above)
;;
*/
Retcode
TBCursor::updateRec()
{
  BCursor::updateRec();
  switch ( lastError )
  {
    case DBIERR_BLOBNOTOPENED : //This BLOB was never opened
    //case PXERR_BLOBNOTOPEN : //951114
      lastError = DBIERR_NONE;
      break;
    case DBIERR_NONE :
    default :
      break;
  }
#if 0//950520
  return ( checkExc() );
#else
  if ( DBIERR_NONE != checkExc() )
  {
    throwExc( "updateRec()" );
  }
  return ( lastError );
#endif//950520
}

/*
;;protected:
;;Retcode
;;TBCursor::updateRec(BRecord *rec)
;;
*/
Retcode
TBCursor::updateRec(BRecord *rec)
{
  return ( BCursor::updateRec( rec ) );
}

/*
;;Retcode
;;TBCursor::updateRec(TBRecord *rec)
;;
;;  Throws a (Retcode) exception on anything but:
;;    DBIERR_NONE
;;    DBIERR_BOF
;;    DBIERR_EOF
;;    DBIERR_FILELOCKED     (likely for this funct.)
;;    DBIERR_KEYVIOL        (likely for this funct.)
;;    DBIERR_RECLOCKFAILED  (likely for this funct.)
;;    PXERR_INVCURRRECORD
;;
;;    (otherwise returns one of the above)
;;
*/
Retcode
TBCursor::updateRec(TBRecord *rec)
{
  BCursor::updateRec( rec );
#if 0//950520
  return ( checkExc() );
#else
  if ( DBIERR_NONE != checkExc() )
  {
    throwExc( "updateRec()" );
  }
  return ( lastError );
#endif//950520
}

/*
;;Retcode
;;TBCursor::updateRec(const void far *buffer, int size)
;;
;;  Throws a (Retcode) exception on anything but:
;;    DBIERR_NONE
;;    DBIERR_BOF
;;    DBIERR_EOF
;;    DBIERR_FILELOCKED     (likely for this funct.)
;;    DBIERR_KEYVIOL        (likely for this funct.)
;;    DBIERR_RECLOCKFAILED  (likely for this funct.)
;;    PXERR_INVCURRRECORD
;;
;;    (otherwise returns one of the above)
;;
*/
Retcode
TBCursor::updateRec(const void far *buffer, int size)
{
  BCursor::updateRec( buffer, size );
#if 0//950520
  return ( checkExc() );
#else
  if ( DBIERR_NONE != checkExc() )
  {
    throwExc( "updateRec()" );
  }
  return ( lastError );
#endif//950520
}

