/*
;;tbdataba.cpp
;;
;;  Class inherited from BDatabase
;;
*/

#pragma   hdrstop

#include  <stdcpp.hpp>
#include  <tdbf.hpp>

#ifdef __DLL__
   #define EXPORT _export
#else
   #define EXPORT
#endif

//Non-member functs
Retcode EXPORT GetOptionalParams(pCHAR szDriver, UINT16 *iFields,
                                 pFLDDesc pfldDesc, pCHAR szData);


/*
;;TBDatabase::TBDatabase( TBEngine *eng,
;;                        DBIOpenMode openMode = dbiREADWRITE,
;;                        DBIShareMode shareMode = dbiOPENSHARED)
;;  : BDatabase( eng, openMode, shareMode )
;;
;;  Constructor for TBDatabase, 'universal' form
;;
;;  Throws exception on failure
;;
*/
TBDatabase::TBDatabase( TBEngine *eng,
                      DBIOpenMode openMode,
                      DBIShareMode shareMode )
  : BDatabase( eng, openMode, shareMode ),
#if 0//950515
    ptEngine_( eng )
#else
    engine( eng )
#endif//950515
{
  if ( DBIERR_NONE != lastError )
  {
#if 0//950520
    throw lastError;
#else
    throwExc( "TBDatabase()", "BDatabase() constructor" );
#endif//950520
  }
}

/*
;;TBDatabase::TBDatabase( TBEngine *eng,
;;                        const char *BDDriver,
;;                        const char *BDAlias = "",
;;                        const char *BDPassword = "",
;;                        DBIOpenMode openMode = dbiREADWRITE,
;;                        DBIShareMode shareMode = dbiOPENSHARED)
;;  : BDatabase( eng, BDDriver, BDAlias, BDPassword,
;;                openMode, shareMode )
;;
;;  Constructor for TBDatabase, explicit form
;;
;;  Throws exception on failure
;;
*/
TBDatabase::TBDatabase( TBEngine *eng,
                        const char *BDDriver,
                        const char *BDAlias,
                        const char *BDPassword,
                        DBIOpenMode openMode,
                        DBIShareMode shareMode )
  : BDatabase( eng, BDDriver, BDAlias, BDPassword,
                openMode, shareMode ),
#if 0//950515
    ptEngine_( eng )
#else
    engine( eng )
#endif//950515

{
  if ( DBIERR_NONE != lastError )
  {
#if 0//950520
    throw lastError;
#else
    throwExc( "TBDatabase()", "BDatabase() constructor", NULL, BDAlias );
#endif//950520
  }
}

/*
;;TBDatabase::TBDatabase( TBEngine *eng,
;;                        const char *BDDriver,
;;                        const char *BDAlias,
;;                        const char *BDPassword,
;;                        const char *BDUserName, //This is the new (SQL) parameter
;;                        DBIOpenMode openMode = dbiREADWRITE,
;;                        DBIShareMode shareMode = dbiOPENSHARED)
;;  : BDatabase( eng, BDDriver, BDAlias, BDPassword,
;;                openMode, shareMode )
;;
;;  Constructor for TBDatabase, explicit form
;;
;;  Throws exception on failure
;;
*/
TBDatabase::TBDatabase( TBEngine *eng,
                        const char *BDDriver,
                        const char *BDAlias,
                        const char *BDPassword,
                        const char *BDUserName, //This is the new (SQL) parameter
                        DBIOpenMode openMode,
                        DBIShareMode shareMode )
  : BDatabase( eng, BDDriver, BDAlias, BDPassword, BDUserName,
                openMode, shareMode ),
#if 0//950515
    ptEngine_( eng )
#else
    engine( eng )
#endif//950515

{
  if ( DBIERR_NONE != lastError )
  {
#if 0//950520
    throw lastError;
#else
    throwExc( "TBDatabase()", "BDatabase() constructor", NULL, BDAlias );
#endif//950520
  }
}

/*
;;virtual
;;TBDatabase::~TBDatabase()
;;
;;  Destructor for TBDatabase
;;
*/
TBDatabase::~TBDatabase()
{
}

/*
;;Retcode
;;TBDatabase::appendTable( const char *srcTable,  //(This database)
;;                         const char *destTable, //(This or other database)
;;                         const char *destType = NULL,
;;                         const TBDatabase &db = NULL,
;;                         eBATMode xBatMode = batAppend )
;;
;;  Changes by //BR to KDBF original
;;    941112
;;      -Made sure destTblDesc is cleared initially
;;
;;  Additions by //BR
;;    941123
;;      -Removed 'xBatMode' arg from BDatabase (moved to TBDatabase class)
;;    941112
;;      -Added 'xBatMode' arg
;;
*/
//BR END
// Append records of a table to the destination table.
Retcode
TBDatabase::appendTable(const char *srcTable,
                        const char *destTable,
                        const char *destType,
                        const TBDatabase &db,
                        eBATMode xBatMode )
{
    BATTblDesc  srcTblDesc;
    BATTblDesc  destTblDesc;

    if (!isOpen)
    {
        return (lastError = PXERR_DBNOTOPEN);
    }

    memset((pVOID)&srcTblDesc, 0, sizeof(BATTblDesc));
//BR BEG
#if 0
    memset((pVOID)&srcTblDesc, 0, sizeof(BATTblDesc));
#else
    memset((pVOID)&destTblDesc, 0, sizeof(BATTblDesc));
#endif//0
//BR END

    // Source table values
    //   Database
    srcTblDesc.hDb = hDb;
    //   tablename
    strcpy(srcTblDesc.szTblName, srcTable);
    //   Table Type
    strcpy(srcTblDesc.szTblType, Driver);

    // Destinarion table
    //   Database
    if (&db == NULL)
    {
        destTblDesc.hDb = hDb;
    }
    else
    {
        destTblDesc.hDb = db.hDb;
    }

    //   Table Name
    strcpy(destTblDesc.szTblName, destTable);

    //   Table Type
    if (destType == NULL)
    {
        strcpy(destTblDesc.szTblType, Driver);
    }
    else
    {
        strcpy(destTblDesc.szTblType, destType);
    }

//BR BEG
#if 0
    lastError = DbiBatchMove(&srcTblDesc, NULL, &destTblDesc, NULL, batAPPEND,
                             NULL, NULL, NULL, NULL, NULL, NULL, NULL,
                             NULL, NULL, NULL, NULL, TRUE, TRUE,
                             NULL, TRUE);
#else
    lastError = DbiBatchMove(&srcTblDesc, NULL, &destTblDesc, NULL, xBatMode,
                             NULL, NULL, NULL, NULL, NULL, NULL, NULL,
                             NULL, NULL, NULL, NULL, TRUE, TRUE,
                             NULL, TRUE);
#endif//0
//BR END

    return lastError;
}

/*
;;Retcode
;;TBDatabase::appendTable(TBCursor* ptCsrSource,        //MUST be Csr of THIS database
;;                       const char *destTableName,   //This or other database
;;                       const char *destType = NULL,
;;                       const TBDatabase &db = NULL,
;;                       eBATMode xBatMode = batAPPEND )
;;
;;  Additions by //BR
;;    11/06/94
;;      -New function: appendTable with BCursor source
;;
*/
Retcode
TBDatabase::appendTable(TBCursor* ptCsrSource,
                       const char *destTable,
                       const char *destType,
                       const TBDatabase &db,
                       eBATMode xBatMode )
{
#if 0
    BATTblDesc  srcTblDesc;
#endif//0
    BATTblDesc  destTblDesc;

    if (!isOpen)
    {
        return (lastError = PXERR_DBNOTOPEN);
    }

#if 0
    memset((pVOID)&srcTblDesc, 0, sizeof(BATTblDesc));
    memset((pVOID)&srcTblDesc, 0, sizeof(BATTblDesc));

    // Source table values
    //   Database
    srcTblDesc.hDb = hDb;
    //   tablename
    strcpy(srcTblDesc.szTblName, srcTable);
    //   Table Type
    strcpy(srcTblDesc.szTblType, Driver);
#else
    memset((pVOID)&destTblDesc, 0, sizeof(BATTblDesc));
#endif//0
    // Destinarion table
    //   Database
    if (&db == NULL)
    {
        destTblDesc.hDb = hDb;
    }
    else
    {
        destTblDesc.hDb = db.hDb;
    }

    //   Table Name
    strcpy(destTblDesc.szTblName, destTable);

    //   Table Type
    if (destType == NULL)
    {
        strcpy(destTblDesc.szTblType, Driver);
    }
    else
    {
        strcpy(destTblDesc.szTblType, destType);
    }
#if 0
    lastError = DbiBatchMove(&srcTblDesc, NULL, &destTblDesc, NULL, batAPPEND,
                             NULL, NULL, NULL, NULL, NULL, NULL, NULL,
                             NULL, NULL, NULL, NULL, TRUE, TRUE,
                             NULL, TRUE);
#else
    lastError = ptCsrSource->gotoBegin();
    if ( DBIERR_NONE != lastError )
    {
      return ( lastError );
    }

    lastError = DbiBatchMove(NULL, ptCsrSource->tabH, &destTblDesc, NULL, xBatMode,
                             NULL, NULL, NULL, NULL, NULL, NULL, NULL,
                             NULL, NULL, NULL, NULL, TRUE, TRUE,
                             NULL, TRUE);
#endif//0

    return lastError;
}

/*
;;Retcode
;;TBDatabase::appendTable(const char* srcTable,   //This database
;;                       TBCursor* ptCsrDest,     //MUST be Csr of THIS database
;;                       eBATMode xBatMode = batAPPEND )
;;
;;  Additions by //BR
;;    11/06/94
;;      -New function: appendTable with BCursor dest.
;;
*/
Retcode
TBDatabase::appendTable(const char *srcTable,
                       TBCursor* ptCsrDest,
                       eBATMode xBatMode )
{
    BATTblDesc  srcTblDesc;
#if 0
    BATTblDesc  destTblDesc;
#endif//0

    if (!isOpen)
    {
        return (lastError = PXERR_DBNOTOPEN);
    }

    memset((pVOID)&srcTblDesc, 0, sizeof(BATTblDesc));
#if 0
    memset((pVOID)&srcTblDesc, 0, sizeof(BATTblDesc));
#endif//0

    // Source table values
    //   Database
    srcTblDesc.hDb = hDb;
    //   tablename
    strcpy(srcTblDesc.szTblName, srcTable);
    //   Table Type
    strcpy(srcTblDesc.szTblType, Driver);

#if 0
    // Destinarion table
    //   Database
    if (&db == NULL)
    {
        destTblDesc.hDb = hDb;
    }
    else
    {
        destTblDesc.hDb = db.hDb;
    }

    //   Table Name
    strcpy(destTblDesc.szTblName, destTable);

    //   Table Type
    if (destType == NULL)
    {
        strcpy(destTblDesc.szTblType, Driver);
    }
    else
    {
        strcpy(destTblDesc.szTblType, destType);
    }
#endif//0

#if 0
    lastError = DbiBatchMove(&srcTblDesc, NULL, &destTblDesc, NULL, batAPPEND,
                             NULL, NULL, NULL, NULL, NULL, NULL, NULL,
                             NULL, NULL, NULL, NULL, TRUE, TRUE,
                             NULL, TRUE);
#else
    lastError = DbiBatchMove(&srcTblDesc, NULL, NULL, ptCsrDest->tabH, xBatMode,
                             NULL, NULL, NULL, NULL, NULL, NULL, NULL,
                             NULL, NULL, NULL, NULL, TRUE, TRUE,
                             NULL, TRUE);
#endif//0

    return lastError;
}

/*
;;Retcode
;;TBDatabase::appendTable(TBCursor* ptCsrSrc,   //MUST be Csr of THIS database
;;                       TBCursor* ptCsrDest,  //MUST be Csr of THIS database
;;                       eBATMode xBatMode = batAPPEND )
;;
;;    11/06/94
;;      -New function: appendTable with BCursor src., dest.
;;
*/
Retcode
TBDatabase::appendTable( TBCursor* ptCsrSrc,
                         TBCursor* ptCsrDest,
                         eBATMode xBatMode )
{
#if 0
    BATTblDesc  srcTblDesc;
    BATTblDesc  destTblDesc;
#endif//0

    if (!isOpen)
    {
        return (lastError = PXERR_DBNOTOPEN);
    }

#if 0
    memset((pVOID)&srcTblDesc, 0, sizeof(BATTblDesc));
    memset((pVOID)&srcTblDesc, 0, sizeof(BATTblDesc));

    // Source table values
    //   Database
    srcTblDesc.hDb = hDb;
    //   tablename
    strcpy(srcTblDesc.szTblName, srcTable);
    //   Table Type
    strcpy(srcTblDesc.szTblType, Driver);

    // Destinarion table
    //   Database
    if (&db == NULL)
    {
        destTblDesc.hDb = hDb;
    }
    else
    {
        destTblDesc.hDb = db.hDb;
    }

    //   Table Name
    strcpy(destTblDesc.szTblName, destTable);

    //   Table Type
    if (destType == NULL)
    {
        strcpy(destTblDesc.szTblType, Driver);
    }
    else
    {
        strcpy(destTblDesc.szTblType, destType);
    }
#endif//0

#if 0
    lastError = DbiBatchMove(&srcTblDesc, NULL, &destTblDesc, NULL, batAPPEND,
                             NULL, NULL, NULL, NULL, NULL, NULL, NULL,
                             NULL, NULL, NULL, NULL, TRUE, TRUE,
                             NULL, TRUE);
#else
    lastError = ptCsrSrc->gotoBegin();
    if ( DBIERR_NONE != lastError )
    {
      return ( lastError );
    }
    lastError = DbiBatchMove(NULL, ptCsrSrc->tabH, NULL, ptCsrDest->tabH, xBatMode,
                             NULL, NULL, NULL, NULL, NULL, NULL, NULL,
                             NULL, NULL, NULL, NULL, TRUE, TRUE,
                             NULL, TRUE);
#endif//0

    return lastError;
}

Retcode
TBDatabase::beginTransaction(hDBIXact &tranHandle, eXILType tranType)
{
#if 0//950520
    if (!isOpen)
    {
        lastError = PXERR_DBNOTOPEN;
        return lastError;
    }

    lastError = DbiBeginTran(hDb, tranType, &tranHandle);

    return lastError;
#else
  BDatabase::beginTransaction( tranHandle, tranType );
  if ( DBIERR_NONE != lastError )
  {
    throwExc( "beginTransaction()" );
  }
  return ( lastError );
#endif//950520
}

/*
;;Retcode
;;TBDatabase::createTable( const char *tableName,
;;                         int numFields, const FieldDesc *fldDescs,
;;                         ValidityDesc *valDesc, RefDesc *refDesc )
;;
*/
Retcode
TBDatabase::createTable( const char *tableName,
                         int numFields, const FieldDesc *fldDescs,
                         ValidityDesc *valDesc, RefDesc *refDesc )
{
  return ( BDatabase::createTable( tableName, numFields, fldDescs, valDesc, refDesc ) );
}

/*
;;Retcode
;;TBDatabase::createTableBorrow( const char* psTblName,
;;                               const char* psBorrowFrom,
;;                               TBDatabase* ptDbFrom )
;;
;;  New function added by //BR
;;
;;  Creates table in home database or specified path, using
;;    structure of the BorrowFrom table (also in the home database)
;;
;;  Throws (Retcode) exception on any error
;;
*/
Retcode
TBDatabase::createTableBorrow( const char* psTblName, const char* psBorrowFrom,
                               TBDatabase* ptDbFrom )
{
  int         nFld;
  FieldDesc*  ptFldDescs = NULL;
  //
  if ( NULL == ptDbFrom )
  {
    ptDbFrom = this;
  }
  lastError = ptDbFrom->getDescVector( (char*) psBorrowFrom, nFld, ptFldDescs );
  if ( DBIERR_NONE != lastError  )
  {
#if 0//950520
    throw lastError;
#else
    throwExc( "createTableBorrow()" );
#endif//950520
  }
  deleteTable( psTblName ); //Delete 1st to ensure createTable doesn't choke
  switch ( lastError )
  {
    case DBIERR_NONE :
    case DBIERR_NOSUCHFILE :
    case DBIERR_NOSUCHTABLE :
      break;
    default :
#if 0//950520
      throw lastError;
#else
      throwExc( "createTableBorrow()" );
#endif//950520
  }
  createTable( psTblName, nFld, ptFldDescs );
  DELETE_A_SAFE( ptFldDescs );
  if ( DBIERR_NONE != lastError )
  {
#if 0//950520
    throw lastError;
#else
    throwExc( "createTableBorrow()" );
#endif//950520
  }
  return ( lastError );
}

/*
;;Retcode
;;TBDatabase::endTransaction( eXEnd tranType )
;;
*/
Retcode
TBDatabase::endTransaction( eXEnd tranType )
{
#if 0//950520
    if (!isOpen)
    {
        lastError = PXERR_DBNOTOPEN;
        return lastError;
    }

    lastError = DbiEndTran(hDb, NULL, tranType);
    return lastError;
#else
  BDatabase::endTransaction( tranType );
  if ( DBIERR_NONE != lastError )
  {
    throwExc ( "endTransaction()" );
  }
  return ( lastError );
#endif//950520
}

/*
;;protected:
;;Retcode
;;TBDatabase::query(const char *qryString, BCursor &answer,
;;                         DBIQryLang qryLang)
;;
*/
Retcode
TBDatabase::query(const char *qryString, BCursor &answer,
                         DBIQryLang qryLang)
{
  return ( BDatabase::query( qryString, answer, qryLang ) );
}

/*
;;Retcode
;;TBDatabase::query(const char *qryString, TBCursor &answer,
;;                         DBIQryLang qryLang)
;;
;;  Use for only for queries returning an answer cursor (select, etc.)
;;    Throws an exception if query does not return an answer cursor
;;
*/
Retcode
TBDatabase::query(const char *qryString, TBCursor &answer,
                         DBIQryLang qryLang)
{
    TABLEHANDLE tabH = 0;

    if (!isOpen)
    {
        return (lastError = PXERR_DBNOTOPEN);
    }

    lastError = DbiQExecDirect(hDb, qryLang, (pCHAR)qryString, &tabH);
        //If tabH == 0 then attach() below will throw an error
    if ( 0 == tabH ) //? No answer cursor returned
    {
      throwExc( "query()", "No expected answer cursor returned" );
    }
    if (lastError)
    {
        return lastError;
    }
    lastError = answer.attach(this, tabH);

    DbiCloseCursor(&tabH);

    return lastError; //All errors returned to caller
}

/*
;;Retcode
;;TBDatabase::queryNoAnswer(const char *qryString,
;;                         DBIQryLang qryLang)
;;
;;  Use only for queries returning no answer cursor (delete, etc.)
;;    Throws an exception if query does return an answer cursor
;;
*/
Retcode
TBDatabase::queryNoAnswer(const char *qryString,
                         DBIQryLang qryLang)
{
    TABLEHANDLE tabH = 0;

    if (!isOpen)
    {
      return (lastError = PXERR_DBNOTOPEN);
    }

    lastError = DbiQExecDirect(hDb, qryLang, (pCHAR)qryString, &tabH);
    if (lastError)
    {
      return lastError;
    }
    if ( 0 != tabH ) //Returned an answer (was not supposed to)
    {
      DbiCloseCursor(&tabH);
      throwExc( "queryNoAnswer()", "Unexpected answer cursor returned" );
    }
    return lastError; //All errors returned to caller
}

/*
;;Retcode
;;TBDatabase::sortTable( const char* psTblname,
;;                       TSortTable& rtSortTable )
;;
;;  Sort (unkeyed, local) table to itself
;;  FieldNumber args
;;
*/
Retcode
TBDatabase::sortTable( const char* psTblname,
                       TSortTable& rtSortTable )
{
  int i;
  for ( i = 0; DBIMAXFLDSINKEY > i; i++ )
  {
    if ( 0 == rtSortTable.ahSortFlds_[ i ] )
    {
      break;
    }
  }
  //i == Num of sort fields
  rtSortTable.nSortFlds_ = i;
  lastError = DbiSortTable( hDb,
                            (pCHAR) psTblname,
                            Driver,
                            NULL,
                            NULL,
                            NULL,
                            NULL,
                            rtSortTable.nSortFlds_,
                            (pUINT16) &rtSortTable.ahSortFlds_,
                            NULL,
                            NULL,
                            NULL,
                            bFalse,
                            NULL,
                            NULL );
  return ( lastError );
}

/*
;;Retcode
;;TBDatabase::sortTable( const char* psTblname,
;;                       TSortTableFldName& rtSortTableFldName )
;;
;;  Sort (unkeyed, local) table to itself
;;  FieldName args
;;
*/
Retcode
TBDatabase::sortTable( const char* psTblname,
                       TSortTableFldName& rtSortTableFldName )
{
  int         i;
  int         h;
  //
#if 0//950504
#else
  {
    //Cursor open only during scope
    TBCursor    tCsr( this, psTblname );
    TBRecord    tRec( &tCsr );
#endif//950504
    for ( i = 0; DBIMAXFLDSINKEY > i; i++ )
    {
#if 0//950504
      //Cursor open only during scope of for loop
      TBCursor    tCsr( this, psTblname );
      TBRecord    tRec( &tCsr );
#endif//950504
      if ( 0 == strlen( rtSortTableFldName.psFldName_[ i ] ) ) //? End of fieldname args
      {
        break;
      }
      h = tRec.getFieldNumber( rtSortTableFldName.psFldName_[ i ] );
      if ( 0 == h ) //? Bad fieldname arg
      {
        lastError = ERRCODE_INVALIDPARAM;
        throw ( lastError );
      }
      rtSortTableFldName.ptSortTable_->ahSortFlds_[ i ] = h;
    }
  }
  //tCsr, tRec released
  return ( sortTable( psTblname, *rtSortTableFldName.ptSortTable_ ) );
}

/*
;;protected:
;;void
;;TBDatabase::throwExc( const char* fnArg,
;;                    const char* errArg = NULL,
;;                    const char* infoArg = NULL,
;;                    const char* aliasArg = NULL )
;;
*/
void
TBDatabase::throwExc( const char* fnArg,
                      const char* errArg,
                      const char* infoArg,
                      const char* aliasArg )
{
  const char* strAlias;
  if ( isOpen ) //Only if isOpen
  {
    strAlias = Alias;
  }
  else
  {
    strAlias = aliasArg;
  }
  TUtil::ThrowExc( fnArg,
                   errArg,
                   infoArg,
                   "TBDatabase",
                   lastError,
                   strAlias );
}

/*
;;TSortTable::TSortTable()
;;
;;  Constructor for TSortTable class
;;
*/
TSortTable::TSortTable()
{
  int i;
  //
  for ( i = 0; DBIMAXFLDSINKEY > i; i++ )
  {
    ahSortFlds_[ i ] = 0;
  }
}

/*
;;TSortTableFldName::TSortTableFldName()
;;
;;  Constructor for TSortTableFldName class
;;
*/
TSortTableFldName::TSortTableFldName()
{
  ptSortTable_ = new TSortTable();
  int i;
  //
  for ( i = 0; DBIMAXFLDSINKEY > i; i++ )
  {
    psFldName_[ i ] = new char[ DBIMAXNAMELEN + 1 ];
    *psFldName_[ i ] = '\0';
  }
}

/*
;;TSortTableFldName::~TSortTableFldName()
;;
;;  Destructor for TSortTableFldName class
;;
*/
TSortTableFldName::~TSortTableFldName()
{
  delete ptSortTable_;
  int i;
  //
  for ( i = 0; DBIMAXFLDSINKEY > i; i++ )
  {
    delete [] psFldName_[ i ];
  }
}

